/*
  Program: Algoritmus pro hran pikvorek pomoc alfa-beta prohledvn
  Autor: Radek Pelnek  
  Ploha ke knize "Jak to vyeit",  www.radekpelanek.cz/?jtv
*/

#include <iostream>
#define X 1
#define O 2
#include <cstdlib>
#include <string>
#include <map>
#define tricode(a1,a2,a3,a4,a5) (a1+a2*3+a3*9+a4*27+a5*81)

using namespace std;

int pole[22][22];
int moje;

// zakdovn 8 smr vyhledvn, abychom pak nemuseli mt 8 if
int smerX[8] = {0, 1, 1, 1, 0, -1, -1, -1};
int smerY[8] = {-1, -1, 0, 1, 1, 1, 0, -1};

void inicializace() {
  srand(time(NULL));
  
  for (int i=0; i<22; i++)
    for (int j=0; j<22; j++)
      pole[i][j] = 0;
}

// naten plnu
void read() {
  char ch;
  cin >> ch;
  if (ch == 'X') moje = X; else moje = O; 

  for (int i=1; i<=20; i++)
    for (int j=1; j<=20; j++) {
      cin >> ch;
      if (ch == 'X') pole[i][j] = X;      
      if (ch == 'O') pole[i][j] = O;
    }
}

// vpis plnu
void print() {
  if (moje == X) cout << "O"; else cout <<"X";
  cout <<endl;
  for (int i=1; i<=20; i++) {
    for (int j=1; j<=20; j++) {
      if (pole[i][j] == X) { cout << "X";}      
      else if (pole[i][j] == O) { cout << "O";}
      else cout << ".";
    }
    cout << endl;
  }
}

char dej_char(int pom) {
  if (pom == X) return 'X';
  if (pom == O) return 'O';
  return '.';
}


void posun(int* x, int* y, int smer, int pocet) {
  *x = *x + smerX[smer]*pocet;  
  *y = *y + smerY[smer]*pocet;
}

bool out(int x, int y) {
  return (x<=0 || y<= 0|| x>20 ||y>20);
}


bool test(int x, int y, int smer, string co) {
  int i = 0;
  while (i < co.length()) {
    if (out(x,y)) return false;
    if (dej_char(pole[x][y]) != co[i]) {      
      return false;
    }
    posun(&x, &y, smer, 1);
    i++;
  }
  return true;
}

bool hledej(string co) {
  for (int i=1; i<=20; i++)
    for (int j=1; j<=20; j++)
      for (int s=0; s<8; s++) {
	if (test(i,j,s,co)) {
	  return true;
	}
      }
  return false;
}

int is_terminal() {
  return (hledej("XXXXX") || hledej("OOOOO"));
}

int positions_evaluated = 0;


// heuristick ohodnocen aktuln pozice
int eval_position(bool mymove) {
  map<int, int> vyskyty;
  // v tto map ukldme poty vskyt jednotlivch vzor
  // kad vzor je kdovn pomoc pevodu ternrnho zpisu na int
  
  int mysymbol = mymove ? moje : 3-moje;
  
  positions_evaluated++;
  for (int i=1; i<=20; i++)
    for (int j=1; j<=20; j++)
      for (int s=0; s<4; s++) { //sta vyhodnocovat 4 smry, zbvajc smry najdeme "z druhho konce"
	int x, y;
	bool valid = true;
	int code = 0, base = 1;
	for (int c=0; c<5; c++) {
	  x = i + smerX[s]*c;
	  y = j + smerY[s]*c;
	  if (out(x,y)) valid = false;
	  int tmp = 0;
	  if (pole[x][y] == mysymbol) tmp = 1;
	  if (pole[x][y] == 3-mysymbol) tmp = 2;	  
	  code += base*tmp;
	  base *= 3;
	}
	if (valid) vyskyty[code]++;
      }
  // podle vskyt jednotlivch vzor vybodujeme heuristick ohodnocen
  int value = 0;  
  value += vyskyty[tricode(1,1,1,1,1)] * 1000; 
  value -= vyskyty[tricode(2,2,2,2,2)] * 1000; 
  value += vyskyty[tricode(0,1,1,1,1)] * 80; 
  value += vyskyty[tricode(1,0,1,1,1)] * 80; 
  value += vyskyty[tricode(1,1,0,1,1)] * 80; 
  value += vyskyty[tricode(1,1,1,0,1)] * 80; 
  value += vyskyty[tricode(1,1,1,1,0)] * 80; 
  value -= vyskyty[tricode(0,2,2,2,2)] * 80; 
  value -= vyskyty[tricode(2,0,2,2,2)] * 80; 
  value -= vyskyty[tricode(2,2,0,2,2)] * 80; 
  value -= vyskyty[tricode(2,2,2,0,2)] * 80; 
  value -= vyskyty[tricode(2,2,2,2,0)] * 80; 
  value += vyskyty[tricode(0,1,1,1,0)] * 120; 
  value -= vyskyty[tricode(0,2,2,2,0)] * 120; 
  value += vyskyty[tricode(0,1,1,0,0)] * 5; 
  value += vyskyty[tricode(0,1,0,1,0)] * 5; 
  value += vyskyty[tricode(0,0,1,1,0)] * 5; 
  value -= vyskyty[tricode(0,2,2,0,0)] * 5; 
  value -= vyskyty[tricode(0,2,0,2,0)] * 5; 
  value -= vyskyty[tricode(0,0,2,2,0)] * 5;
  return value;
}

bool reasonable_position(int x, int y) {
  if (pole[x][y] != 0) return false;
  for (int i=-1; i<=1; i++)
    for (int j=-1; j<=1; j++)
      if ((i!=0 ||j!=0) && pole[x+i][y+j]!=0) return true;
  return false;
}

void do_move(int x, int y, bool mymove) {
  if ((mymove && moje == X) || (!mymove && moje == O)) {
    pole[x][y] = X;
  } else {
    pole[x][y] = O;
  }
}

void undo_move(int x, int y) {
  pole[x][y] = 0;
}

int movex, movey;  // optimln dosud nalezen tah udrujeme v globln promnn
int maxdepth;

// alfa-beta prohledvn
// vylepen algoritmu minimax o oezvn
int alphabeta(int depth, bool mymove, int alpha, int beta) {
  if (depth == maxdepth || is_terminal()) {
    return eval_position(mymove);
  } else {
    for (int i=1; i<=20; i++)
      for (int j=1; j<=20; j++)
	if (reasonable_position(i,j)) {
	  do_move(i,j,mymove);
	  int tmp = -alphabeta(depth+1, !mymove, -beta, -alpha);
	  if (tmp > alpha) {
	    alpha = tmp;
	    if (depth==0) {movex = i; movey= j; }
	  }
	  undo_move(i,j);
	  if (beta <= alpha) return alpha;
	}    
  }
  return alpha;  
}

// nhodn vbr tahu
void vyber_dle_strategie_random(int* x, int* y) {
  int odchylka = 1;
  do {
    *x = (rand()%(2*odchylka))+10-odchylka;
    *y = (rand()%(2*odchylka))+10-odchylka;
    if (odchylka <9) odchylka++;
  } while (pole[*x][*y] != 0);
}

void decide() {
  // fixn voln s hloubkou 3
  // mon vylepen: opakovan volat s rostouc hloubkou  
  maxdepth = 3;
  movex = -1; movey = -1;
  int result = alphabeta(0,true, -10000,+10000);
 
  // pokud alfa-beta nenajde dn tah (co nastv pouze pi przdnm plnu)
  // zahrajeme nhodn
  if (movex == -1) 
    vyber_dle_strategie_random(&movex, &movey);
  pole[movex][movey] = moje;
}

int main() {

  inicializace();  
  read();
  decide();
  print();

  return 0;
}
